/*  Specification of class galimber which implements rotation and projection of
 *  an object onto a plane such that hidden lines are eliminated.
 */
#include <math.h>
#include <stdlib.h>
#include <deque>
using namespace std;

#define EOF (-1)

//Line segment to be queued for galimber::project() output
class line
{
  public:
    float xstart;
    float ystart;
    float xend;
    float yend;
    line(float xstart, float ystart, float xend, float yend):
        xstart(xstart),ystart(ystart),xend(xend),yend(yend){}
};

//The galimber hidden line eliminator class
class galimber
{
  public:
    //Constructor
    galimber(int nump, double p[][3], int numf, double face[][20]);
    //Merthod to do HLE projection
    void project(double ipsi, double ithe, double iphi, deque<line> &linequeue);
  private:
    void rotate(double psi, double theta, double phi, double dircos[3][3]);
    void xformr(double in[3], double dircos[3][3], double out[3]);
    void error(char *errclass); //Print error class
    
    bool debug; //Print extensive debugging info for each projection if true
    
    short nump; //Number of points
    double p[200][3]; //Number of points (nodes)
    double v[200][3]; //Vertices (projected nodes)
    short vnatur[200][20]; //Nature of each vertex

    short numf; //Number of faces
    short icopla; //Number of coplaner faces
    short face[200][21]; //Clockwise points corresponding to each face
    double normal[200][3]; //Unit inward normal to each face
    bool onview[200]; //Onview factor for each face

    short nume; //Number of edges
    short edgeno[200][2]; //2 nodes corresponding to each edge
    short edgefa[200][2]; //2 faces corresponding to each edge
    double concav[200]; //Concavity factor for each edge
    short eintse[200][20]; //Winning edge at losing intersection

    short intensity[200]; //Segment intensity
    short isecnt[200]; //Segment intersection count

    double pintse[200][20]; //Losing intersection points
    int maxdim; //Maximum dimension of any rotated point (used for scaling)
 };
